/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import java.util.*;

import org.w3c.dom.*;


/**
 * Children (besides the inherited):<br>
 *   value<br>
 */
public abstract class A_SingleValueField extends A_Field 
{
	private String value;
	
	public A_SingleValueField(Form form) {
		super(form);
	}
	
	public A_SingleValueField(Form form, Node field) {
		super(form, field);
	}
	
	public List getOptions(){
		return null;
	}
	
	public void setOptions(List opts) {
		return;
	}
	
	public int cardinality() {
		return I_Field.CARDINALITY_ONE;
	}

	/**
	 * Look for relevant sub-elements
	 * @param child
	 * @param childType
	 * @param childName
	 * @return true if child is considered handled, false if it didn't cause any changes
	 */
	public boolean handleChildNode(Node child, short childType, String childName)
	{
		if(super.handleChildNode(child, childType, childName))
			return true;
		if(childType == Node.ELEMENT_NODE) {
			if (childName.equals(Form.FBTAG_VALUE)) {
				setValue(Form.collectText(child));
				return true;
			}
		}
		return false;
	}
	
	//
	// accessors
	//

	public String getValue() {
		return value;
	}
	
	public void setValue(String str) {
		this.value = str;
	}

	
	/**
	 * Interface requieres this to work even if cardinality is one.
	 * @return a List with one item, the value
	 */
	public List getValues() {
		if(value != null) {
			List l = new ArrayList();
			l.add(value);
			return l;
		} else {
			return null;
		}
	}

	/**
	 * Sets the value to the first item of the given list, if the list is empty this is the same as setting the value to null
	 * @param List of values
	 */
	public void setValues(List values) {
		if(values == null || values.size() == 0)
			setValue(null);
		else
			setValue((String)values.get(0));
	}
	
	/**
	 * Sets the value to the first item of the given array, if the array is empty this is the same as setting the value to null
	 * @param Array of values
	 */
	public void setValues(String[] values) {
		if(values == null || values.length == 0)
			setValue(null);
		else
			setValue(values[0]);
	}

	//
	//
	//
	
	public String childrenToXML(String indent, boolean inclVal) 
	{
		String tmp = super.childrenToXML(indent, inclVal);
	    if(tmp == null && getValue() == null)
	    	return null;
		StringBuffer b = new StringBuffer();
		if(tmp != null)
			b.append(tmp);
		if(getValue() != null)
			b.append("\n" + indent + "<"+Form.FBTAG_VALUE+">" + getValue() + "</"+Form.FBTAG_VALUE+">");
 		return b.toString();
	}
}
