/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 182 $
 * $LastChangedDate: 2007-07-05 11:13:28 +0200 (Thu, 05 Jul 2007) $
 * 
 */

/*
 * Example:
 * 
<form>
	<ident></ident>
	<fields>
		<raw></raw>
		<text>
			<ident></ident>
			<name></name>
			<label></label>
			<value></value>
			<format>
				<type>date,number,integer,decimal</type>
				<pattern></pattern>
			</format>
			<validators>
				<required>
					<error></error>
				</required>
				<email>
					<error></error>
				</email>
				<regex>
					<pattern></pattern>
				</regex>
				<options>
					<min></min>
					<max></max>
				</options>
				<range>
					<min></min>
					<max></max>
				</range>
			</validators>
			<layout>
				<style></style>
				<styleName></styleName>
				<stylePrefix></stylePrefix>
				<formAction></formAction>
				<formMethod></formMethod>
				<formName></formName>
				<formBegin></formBegin>
				<formEnd></formEnd>
				<labelBegin></labelBegin>
				<labelEnd></labelEnd>
				<fieldBegin></fieldBegin>
				<fieldEnd></fieldEnd>
				<optionBegin></optionBegin>
				<optionEnd></optionEnd>
				<rows></rows>
				<cols></cols>
			</layout>
			<readonly />
		</text>
		<hidden>
		</hidden>
		<textarea>
		</textarea>
		<checkbox>
			<ignore></ignore>
			<option>
				<label></label>
				<value></value>
			</option>
			<option></option>
		</checkbox>
		<radiolist>
		</radiolist>
		<popuplist>
		</popuplist>
		<selectlist>
			<value></value>
			<value></value>
			<option></option>
			<option></option>
			<option></option>
		</selectlist>
		<checklist>
		</checklist>
		<submit>
		</submit>
		<reset>
		</reset>
	</fields>
	<layout>
	</layout>
	<locale></locale>
	<paramEnc></paramEnc>
</form>	
*/

package com.codedroids.forms;

import java.util.*;
import java.io.*;
import javax.servlet.http.*;
import com.codedroids.j.MapUtils;
import com.codedroids.j.ListUtils;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import com.codedroids.records.I_Record;
import com.codedroids.records.RecordField;
import java.text.ParseException;


public class Form {
	
	public static final String FILE_EXT = ".form.xml";
	
	public static final String FBTAG_BANNER = "banner";
	public static final String FBTAG_BUTTON = "button";
	public static final String FBTAG_CHECK = "checkbox";
	public static final String FBTAG_CHECKLIST = "checklist";
	public static final String FBTAG_COLS = "cols";
	public static final String FBTAG_EMAIL = "email";
	public static final String FBTAG_PARAM_ENC = "paramEnc";
	public static final String FBTAG_ERROR = "error";
	public static final String FBTAG_FIELD_BEGIN = "fieldBegin";
	public static final String FBTAG_FIELD_END = "fieldEnd";
	public static final String FBTAG_FIELDS = "fields";
	public static final String FBTAG_FORM = "form";
	public static final String FBTAG_FORM_ACTION = "formAction";
	public static final String FBTAG_FORM_BEGIN = "formBegin";
	public static final String FBTAG_FORM_END = "formEnd";
	public static final String FBTAG_FORM_METHOD = "formMethod";
	public static final String FBTAG_FORM_NAME = "formName";
	public static final String FBTAG_FORMAT = "format"; // TODO
	public static final String FBTAG_GROUP = "group"; // TODO
	public static final String FBTAG_HIDDEN = "hidden";
	public static final String FBTAG_IDENT = "ident";
	public static final String FBTAG_IGNORE = "ignore";
	public static final String FBTAG_LABEL = "label";
	public static final String FBTAG_LABEL_BEGIN = "labelBegin";
	public static final String FBTAG_LABEL_END = "labelEnd";
	public static final String FBTAG_LABEL_POS = "labelPos";
	public static final String FBTAG_LAYOUT = "layout";
	public static final String FBTAG_MAX = "max";
	public static final String FBTAG_MESSAGE = "message";
	public static final String FBTAG_MIN = "min";
	public static final String FBTAG_NAME = "name";
	public static final String FBTAG_OFF_VALUE = "offValue";
	public static final String FBTAG_ON_VALUE = "onValue";
	public static final String FBTAG_OPTION = "option";
	public static final String FBTAG_OPTION_BEGIN = "optionBegin";
	public static final String FBTAG_OPTION_END = "optionEnd";
	public static final String FBTAG_OPTIONS = "options";
	public static final String FBTAG_PASSWORD = "password";
	public static final String FBTAG_PATTERN = "pattern";
	public static final String FBTAG_POPUP = "popuplist";
	public static final String FBTAG_PRIVATE = "private";
	public static final String FBTAG_POSTSCRIPT = "ps";
	public static final String FBTAG_POSTSCRIPT_BEGIN = "psBegin";
	public static final String FBTAG_POSTSCRIPT_END = "psEnd";
	public static final String FBTAG_RADIO = "radiolist";
	public static final String FBTAG_READONLY = "readonly";
	public static final String FBTAG_REGEX = "regex";
	public static final String FBTAG_RENDER = "render";
	public static final String FBTAG_REQUIRED = "required";
	public static final String FBTAG_RAW = "raw";
	public static final String FBTAG_RESET = "reset";
	public static final String FBTAG_ROWS = "rows";
	public static final String FBTAG_SELECTLIST = "selectlist";
	public static final String FBTAG_STYLE = "style";
	public static final String FBTAG_STYLE_NAME = "styleName";
	public static final String FBTAG_STYLE_PREFIX = "stylePrefix";
	public static final String FBTAG_SUBMIT = "submit";
	public static final String FBTAG_SUBMIT_LIST = "submitlist";
	public static final String FBTAG_TEXT = "text";
	public static final String FBTAG_TEXTAREA = "textarea";
	public static final String FBTAG_TYPE = "type";
	public static final String FBTAG_VALIDATORS = "validators";
	public static final String FBTAG_VALUE = "value";
	
	public static final String FBATTR_LOCALE = "locale";
	
	public static final String PP_INDENT = "    ";
	
	/**
	 * Returns an instance from an already parsed XML file
	 * @param xmlfile
	 * @return
	 */
	public static synchronized Form getInstance(Element root) throws ParserConfigurationException, SAXException, IOException
	{
		Form retval = new Form();
		retval.parseForm(root);
		return retval;
	}
	
	/**
	 * Collect text by concatenating all TEXT and CDATA-nodes into string
	 * @param node
	 * @return String containing the text (may be null)
	 */
	public static String collectText(Node node) {
		return collectText(node, null);
	}
	/**
	 * Collect text by concatenating all TEXT and CDATA-nodes into string
	 * @param node
	 * @param def Default value to be used if no text was found
	 * @return String containing the text or <code>def</code>
	 */
	public static String collectText(Node node, String def) {
		Node child = node.getFirstChild();
		StringBuffer retval = new StringBuffer();
		while(child != null) {
			switch(child.getNodeType()) {
				case Node.TEXT_NODE:
				case Node.CDATA_SECTION_NODE:
					retval.append(child.getNodeValue());
			}
			child = child.getNextSibling();
		}
		if(retval.length()==0)
			return def;
		else
			return retval.toString();
	}
	
	/**
	 * Make string into valid format for HTML tag attributes
	 * @param src
	 * @return
	 */
	public static String escapeAttribute(String source) 
	{
		int l = source.length();
		StringBuffer buf = new StringBuffer((int)(l*1.10));
		for(int i=0; i<l; i++) {
			char ch = source.charAt(i);
			if('&' == ch)
				buf.append("&amp;");
			else if('"' == ch)
				buf.append("&quot;");
			else
				buf.append(ch);
		}
		return buf.toString();
	}
 
	/**
	 * Wraps string as CDATA if needed (currently always if not empty)
	 * @param str
	 * @return
	 */
	public static String cdata(String str) {
		if(str==null || str.length() == 0)
			return str;
		else
			return "<![CDATA[" + str + "]]>";
	}
	
	//
	// Instance
	//
	
	private String ident;
	private List fields;
	private Layout layout;
	private boolean readOnly;
	private Locale localeInstance; // These two are sync'ed whenever the locale is changed -
	private String locale;         // setting it will determine what <label>, <ps> and <error> is returned
	private String paramEnc;
	private I_FormRender formRender;
	private Map fieldRenders = new HashMap(); // Map of I_FieldRender's
	
	// 
	// constructors
	//
	
	/**
	 * 
	 */
	public Form() {
		super();
		setLocale(Locale.getDefault()); // make sure that some locale is defined
		// setup the default renders
		try {
			registerFormRender("com.codedroids.forms.DefaultFormRender");
			registerFieldRender("default", "com.codedroids.forms.DefaultFieldRender");
			registerFieldRender("none", "com.codedroids.forms.NoneFieldRender");
		} catch (Exception e) {
			System.err.println("Failed to register default renders: ");
			e.printStackTrace();
		}
	}
	
	//
	// accessors
	//
	
	public String getIdent() {
		return ident;
	}

	public void setIdent(String str) {
		this.ident = str;
	}
	
	public boolean isReadOnly() {
		return readOnly;
	}
	
	public String getReadOnly() {
		return (readOnly ? "true" : "false");
	}
	
	public void setReadOnly(String str) {
		if(str == null || str.length()==0) {
			this.readOnly = true; // empty field means activate it
		} else {
			try {
				this.readOnly = Boolean.valueOf(str).booleanValue();
			} catch (Exception e) {
				this.readOnly = false;
			}
		}
	}

	public void setReadOnly(boolean b) {
		this.readOnly = b;
	}
	
	public Layout getLayout() {
		return layout;	
	}

	public void setLayout(Layout str) {
		this.layout = str;
	}
	
	public String getLocale() {
		return locale;
	}

	/**
	 * Examples: "da" or "en_US"
	 * @param str
	 */
	public void setLocale(String str) {
		if(str!=null) {
			String[] p = str.split("_");
			if(p.length==1)
				localeInstance = new Locale(p[0]);
			else if(p.length==2)
				localeInstance = new Locale(p[0], p[1]);
			else if(p.length==3)
				localeInstance = new Locale(p[0], p[1], p[2]);
			else
				localeInstance = Locale.getDefault(); // could throw, but for now just do something reasonable
		} else {
			localeInstance = Locale.getDefault();
		}
		locale = localeInstance.toString();
	}
	
	public void setLocale(Locale loc) {
		localeInstance = loc;
		locale = localeInstance.toString();
	}
	
	public Locale locale() {
		return localeInstance;
	}
	
	public String getParamEnc() {
		return paramEnc;	
	}

	public void setParamEnc(String str) {
		this.paramEnc = str;
	}
	
	public List getFields() {
		return fields;
	}

	public void setFields(List str) {
		this.fields = str;
	}
	
	public void addField(I_Field f) {
		if(this.fields == null)
			this.fields = new ArrayList();
		this.fields.add(f);
	}
	
	public void removeField(I_Field f) {
		if(this.fields != null)
			this.fields.remove(f);
	}
	
	public I_FormRender getFormRender() {
		return formRender;
	}

	public void setFormRender(I_FormRender str) {
		this.formRender = str;
	}
	
	public void registerFormRender(String className) throws ClassNotFoundException, IllegalAccessException, InstantiationException {
		setFormRender((I_FormRender)Class.forName(className).newInstance());
	}
	
	public I_FieldRender getFieldRender(String name) {
		return (I_FieldRender)fieldRenders.get(name);
	}

	public void setFieldRender(String name, I_FieldRender r) {
		if(r==null)
			fieldRenders.remove(name);
		else
			fieldRenders.put(name, r);
	}

	public void registerFieldRender(String name, String className) throws ClassNotFoundException, IllegalAccessException, InstantiationException {
		setFieldRender(name, (I_FieldRender)Class.forName(className).newInstance());
	}
	
	//
	// convenience
	// 
	
	public I_Field getFieldByIdent(String ident) {
		if(this.fields != null) {
			Iterator it = fields.iterator();
			while(it.hasNext()){
				I_Field f = (I_Field)it.next();
				if(ident.equals(f.getIdent()))
					return f;
			}
		}
		return null;
	}
	
	public I_Field getFieldByName(String name) {
		if(this.fields != null) {
			Iterator it = fields.iterator();
			while(it.hasNext()){
				I_Field f = (I_Field)it.next();
				if(name.equals(f.getName()))
					return f;
			}
		}
		return null;
	}
	
	public List getFieldsByName(String name) {
		List l = new ArrayList();
		if(this.fields != null) {
			Iterator it = fields.iterator();
			while(it.hasNext()){
				I_Field f = (I_Field)it.next();
				if(name.equals(f.getName()))
					l.add(f);
			}
		}
		return l;
	}
	
	public Set getFieldNames() {
		if(this.fields != null) {
			Set set = new HashSet();
			Iterator it = fields.iterator();
			while(it.hasNext()){
				I_Field f = (I_Field)it.next();
				set.add(f.getName());
			}
			return set;
		}
		return null;
	}

	/**
	 * 
	 */
	private static class KVEntry implements Map.Entry {
		private Object key;
		private Object value;
		public KVEntry(Object key, Object value) {
			this.key = key;
			this.value = value;
		}
		public Object getKey() { return key; }
		public Object getValue() { return value; }
		public Object setValue(Object ignored) { /* No Op. */ return value; } 
	}
	
	/**
	 * The map is populated with the values from the form. The keys correspond to the field names.
	 * If cardinality is larger than 1 the value is a List (which may be empty). Otherwise it is the
	 * values itself (fields with cardinality = 0 are ignored).
	 * -- eventually this should just set the data carrying object. 
	 * @return map with values
	 */
	public Map getValues() {
		if(this.fields != null) {
			Map map = new HashMap();
			Iterator it = fields.iterator();
			while(it.hasNext()){
				I_Field f = (I_Field)it.next();
				String name = f.getName();
				if (name != null) {
					if(f.cardinality() == I_Field.CARDINALITY_ONE) {
						String value = f.getValue();
						map.put(name, value);
					}
					else if(f.cardinality() == I_Field.CARDINALITY_MULTIPLE) {
						List values = f.getValues();
						map.put(name, values);
						
					}
				}
			}
			return map;
		} else {
			return null;
		}
	}
	
    /**
     * Set a map which must follow the above definition. Specifically List should only be present for fields
     * with cardinality > 1.
     * -- eventually this should just return the data carrying object
     * @param values
     */
	public void setValues(Map values) {
		setFieldValuesFromMap(values, null, true, true);
	}
	
	/**
	 * Collects values (as List, even if there is only one value for a given field) and stores them
	 * under the field name in the returned map.
	 * @param filter - if not null only the field names in the set is considered, if null all fields are included
	 * @param exclPrefix - if not null filters having the prefix are exlucdes as well
	 * @return null if no fields are defined, otherwise a map (which may be empty if no values are present)
	 */
	public Map getFieldValues(Set filter, String exclPrefix) {
		if(this.fields != null) {
			Map map = new HashMap();
			Iterator it = fields.iterator();
			while(it.hasNext()){
				I_Field f = (I_Field)it.next();
				List values = f.getValues();
				if(values != null) {
					String name = f.getName();
					if(name != null && (exclPrefix == null || !name.startsWith(exclPrefix)) && (filter == null || filter.contains(name)))
						map.put(name, values);
				}
			}
			return map;
		} else {
			return null;
		}
	}
	// Legacy support
	public Map getFieldValues(Set filter) {
		return getFieldValues(filter, null);
	}
	
	/**
	 * Same as above, but multiple values are stored as a string rather than a List (combining the elements
	 * with the given separators.
	 * @param filter
	 * @param exclPrefix - if not null filters having the prefix are exlucdes as well
	 * @param separator
	 * @param lastSeparator
	 * @return
	 */
	public Map getFieldValues(Set filter, String exclPrefix, String separator, String lastSeparator) {
		if(this.fields != null) {
			Map map = new HashMap();
			Iterator it = fields.iterator();
			while(it.hasNext()){
				I_Field f = (I_Field)it.next();
				List values = f.getValues();
				if(values != null) {
					String name = f.getName();
					if(name != null && (exclPrefix == null || !name.startsWith(exclPrefix)) && (filter == null || filter.contains(name)))
						map.put(name, ListUtils.concat(values, separator, lastSeparator));
				}
			}
			return map;
		} else {
			return null;
		}
	}
	// Legacy support
	public Map getFieldValues(Set filter, String separator, String lastSeparator) {
		return getFieldValues(filter, null, separator, lastSeparator);
	}
	
	/**
	 * Ditto, but returns a List instead of a map (entries are Map.Entry objects)
	 * @param filter - if not null only the field names in the set is considered, null indicates no filtering
	 * @param exclPrefix - if not null filters having the prefix are exlucdes as well
	 * @param separator
	 * @param lastSeparator
	 * @return
	 */
	public List getFieldValuesAsList(Set filter, String exclPrefix, String separator, String lastSeparator) {
		if(this.fields != null) {
			List list = new ArrayList();
			Iterator it = fields.iterator();
			while(it.hasNext()){
				I_Field f = (I_Field)it.next();
				List values = f.getValues();
				if(values != null) {
					String name = f.getName();
					if(name != null && (exclPrefix == null || !name.startsWith(exclPrefix)) && (filter == null || filter.contains(name)))
						list.add(new KVEntry(name, ListUtils.concat(values, separator, lastSeparator)));
				}
			}
			return list;
		} else {
			return null;
		}	
	}
	// Legacy support
	public List getFieldValuesAsList(Set filter, String separator, String lastSeparator) {
		return getFieldValuesAsList(filter, null, separator, lastSeparator);
	}

	/**
	 * Pretty-prints the field values as text using the field labels rather than the field names
	 * @param filter - if not null only the field names in the set is considered, if null all fields are included
	 * @param exclPrefix - if not null filters having the prefix are exlucdes as well
	 * @return
	 */
	public String getFieldValuesAsLabeledText(Set filter, String exclPrefix) {
		List list = getFieldValuesAsList(filter, exclPrefix, ", "," & ");
		List result = new ArrayList(list.size());
		Iterator i = list.iterator();
		while(i.hasNext()) {
			Map.Entry e = (Map.Entry)i.next();
			String fieldName = (String)e.getKey();
			String label = ((A_Field)getFieldByName(fieldName)).getLabel();
			if(label==null)
				label=fieldName;
			Map.Entry n = new KVEntry(label, e.getValue());
			result.add(n);
		}
		return MapUtils.prettyPrint(result);
	}
	
	/**
	 * Pretty-prints the field values as text using the field names as labels
	 * @param filter - if not null only the field names in the set is considered, if null all fields are included
	 * @param exclPrefix - if not null filters having the prefix are exlucdes as well
	 * @return
	 */
	public String getFieldValuesAsText(Set filter, String exclPrefix) {
		List list = getFieldValuesAsList(filter, exclPrefix, ", "," & ");
		return MapUtils.prettyPrint(list);
	}
	// Legacy support
	public String getFieldValuesAsText(Set filter) {
		return getFieldValuesAsText(filter, null);
	}

	//
	// form processing
	//
	
	public String submitCheckField() 
	{
		return "__submit";
	}
	
	/**
	 * Checks for a special parameter to see if the form has been submitted to the code or it is the
	 * first time on a page.
	 * @param request
	 */
	public boolean isSubmitted(HttpServletRequest request) 
	{
		return "true".equals(request.getParameter(submitCheckField()));
	}
	
	public boolean isNotSubmitted(HttpServletRequest request) 
	{
		return !isSubmitted(request);
	}
	
	public String paramEncField() 
	{
		return "__encch";
	}

	public String paramEncString() 
	{
		return "\u00E6";
	}
	
	public boolean needsConversion(HttpServletRequest request) 
	{
		if(getParamEnc() != null) {
			String v = request.getParameter(paramEncField());
			if(v!=null && !v.equals(paramEncString()))
				return true;
		}
		return false;
	}
	public String[] paramConversion(String[] p, String enc) 
	{
		if(p != null) {
			for(int i = 0; i<p.length; i++) {
				try {
					String con = new String(p[i].getBytes(enc), "UTF-8");
					p[i] = con;
				} catch (UnsupportedEncodingException e) {
					// best to leave it unaltered.
				}
			}
		}
		return p;
	}
	
	/**
	 * Collect the parameter values from the request, also deals with the issues surrounding
	 * multiple submit-button values.
	 * <p>Read-only fields will never be set by this method. This is an extra line of defence - even if
	 * someone hacked a request such the otherwise disabled fields comes with altered values they will 
	 * never be set in the form. Note other methods does not provide the same
	 * @param request
	 */
	public void collectValues(HttpServletRequest request) 
	{
		String pEnc = getParamEnc();
		boolean convert = needsConversion(request);
		Map params = request.getParameterMap();
		Iterator it = fields.iterator();
		while(it.hasNext()){
			I_Field f = (I_Field)it.next();
			if(f.isReadOnly())
				continue;
			int card = f.cardinality();
			String name = null;
			if(card == I_Field.CARDINALITY_NONE || (name = f.getName()) == null)
				continue;
			String[] values = null;
			if(f.tagName().equals(FBTAG_SUBMIT_LIST)) {
				// Use the name to get the right value from the relevant option
				Iterator opts = f.getOptions().iterator();
				int ndx = 0;
				while(opts.hasNext()) {
					Option opt = (Option)opts.next();
					String oname = name + (ndx==0 ? "" : "." + ndx);
					values = (String[])params.get(oname);
					if(values != null && values.length > 0) {
						if(convert)
							values = paramConversion(values,pEnc);
						values[0] = opt.getValue();
						break;
					}
					ndx++;
				}
			} else {
				values = (String[])params.get(name);
				if(convert && values != null && values.length > 0)
					values = paramConversion(values,pEnc);
			}
			f.setValues(values);
		}
	}
	/**
	 * Not a reset function since we do not currently have info on the original field values,
	 * so this sets them all to null.
	 * @param inclFields is a Set of fields to include in the result, if null all fields are included.
	 */
	public void setFieldValuesToNull(Set inclFields) {
		setFieldValuesFromMap(new HashMap(), inclFields, true, true);
	}
	/**
	 * Sets the field values according to the contents of the given <code>params</code>.<br>
	 * The sequence of the last three parameters indicate
	 * their priority (i.e. inclFields has the highest priority while inclNull only comes into play if
	 * the other two "filters" did not exclude the field)
	 * @param params a Map of Lists/Arrays, Strings or nulls.
	 * @param inclFields is a Set of fields to include in the result, if null all fields are included.
	 * @param overwrite decides whether an already set(not null) field value is to be replaced or kept
	 * @param inclNull determines if params yields null for a given field will cause the field to be set or it will be unchanged
	 */
	public void setFieldValuesFromMap(Map params, Set inclFields, boolean overwrite, boolean inclNull) {
		if(this.fields != null) {
			Iterator it = fields.iterator();
			while(it.hasNext()){
				I_Field f = (I_Field)it.next();
				String name = f.getName();
				if(name == null || (inclFields != null && !inclFields.contains(name)))
					continue;
				int card = f.cardinality();
				if(card == I_Field.CARDINALITY_NONE)
					continue;
				if(!(overwrite || f.getValues()==null || f.getValues().size()==0))
					continue;
				Object value = params.get(name);
				if(value == null && inclNull) {
					if(card == I_Field.CARDINALITY_ONE)
						f.setValue(null);
					else
						f.setValues((List)null);
				}
				else if(value instanceof List) {
					f.setValues((List)value);
				} 
				else if(value instanceof String[]) {
					f.setValues((String[])value);
				} 
				else {
					f.setValue((String)value);
				}
			}
		}
	}
	/**
	 * Sets the field values according to the contents of the given map
	 * <br>
	 * @param params a Map of Lists, Strings or nulls.
	 * @param inclFields  is a Set of fields to include in the result, if null all fields are included.
	 * @param inclNull determines if params holds null for field cause the field to be set
	 * @param overwrite decides whether an already set(not null) field value is to be replaced or kept
	 * @return a map with values like String[] or null
	 * @deprecated use setFieldValuesFromMap()
	 */
	public void setFieldValueMap(Map params, Set inclFields, boolean inclNull, boolean overwrite) 
	{
		setFieldValuesFromMap(params, inclFields, overwrite, inclNull);
	}
	
	/**
	 * Returns the values of the fields as a map of String[] similar to 
	 * HttpServletRequest.getParameterMap() if inclNull is false. If it is true
	 * then unset fields are entered into the map with a null value
	 * <br>
	 * @param inclFields  is a Set of fields to include in the result, if null all fields are included.
	 * @param inclNull determines if fields with the value = null are included in the returned map
	 * @param forceLists if true even single value are stored in List containers
	 * @return a map with values like List, null (if inclNull) and Strings (forceList is false)
	 */
	public Map getFieldValueMap(Set inclFields, boolean inclNull, boolean forceLists) 
	{
		if(this.fields != null) {
			Map map = new HashMap();
			Iterator it = fields.iterator();
			while(it.hasNext()){
				I_Field f = (I_Field)it.next();
				String name = f.getName();
				if(name == null || (inclFields != null && !inclFields.contains(name)))
					continue;
				int card = f.cardinality();
				switch(card) {
				case I_Field.CARDINALITY_NONE:
					// do nothing
					break;
				case I_Field.CARDINALITY_ONE:
					String value = f.getValue();
					if(value == null && inclNull) {
						map.put(name, null);
					} else {
						if(forceLists)
							map.put(name, Collections.singletonList(value));
						else
							map.put(name, value);
					}
					break;
				case I_Field.CARDINALITY_MULTIPLE:
				default:
					List values = f.getValues();
				    if(values == null || values.size() < 1) {
						if(inclNull)
							map.put(name, null);
				    } else {
						map.put(name, (String[])values.toArray(new String[values.size()]));
				    }
					break;
				}
			}
			return map;
		} else {
			return null;
		}
	}
	
	/**
	 * Expects the values to be String[], use get/setFieldValueMap when working with Lists rather than String[]
	 * @deprecated use collectValues instead (or setFieldValueMap if you need to populate from a map)
	 * @param params
	 * @param overwrite - if true always overwrite value, if false only set the value if existing value is null/empty
	 */
	public void setFieldValues(Map params, boolean overwrite) 
	{
		Iterator it = fields.iterator();
		while(it.hasNext()){
			I_Field f = (I_Field)it.next();
			int card = f.cardinality();
			String name = null;
			if(card == I_Field.CARDINALITY_NONE || (name = f.getName()) == null)
				continue;
			String[] values = null;
			if(name.equals(FBTAG_SUBMIT_LIST)) {
				// Use the name to get the right value from the relevant option
				Iterator opts = f.getOptions().iterator();
				int ndx = 0;
				while(opts.hasNext()) {
					Option opt = (Option)opts.next();
					String oname = name + (ndx++==0 ? "" : "." + ndx);
					values = (String[])params.get(oname);
					if(values != null && values.length > 0) {
						values[0] = opt.getValue();
						break;
					} else if(values!=null) {
						values[0] = null;
					}
				}
			} else {
				values = (String[])params.get(name);
			}
			if(overwrite || f.getValues()==null || f.getValues().size()==0)
				f.setValues(values);
		}
	}
	
	/**
	 * Sets the values of the named field. If field holds any values and <code>overwrite</code> is <code>true</code>
	 * all previous values are removed and the field is set to the <code>values</code>.
	 * <p>If the named field is a single value type of field only the first item of the list is used.
	 * 
	 * @param field name of field
	 * @param values array of string that makes up the values to be set for the field.
	 * @param overwrite if false, the values are only set if the field has no existing values
	 */
	public void setFieldValue(String field, String[] values, boolean overwrite)
	{
        I_Field f = getFieldByName(field);
		if(f != null) {
			int card = f.cardinality();
			if(card == I_Field.CARDINALITY_NONE )
				return;
			if(overwrite || f.getValues()==null || f.getValues().size()==0)
				f.setValues(values);
		}
	}
	
	/**
	 * Sets the value of the named field.
	 * <p> If the named field is a multi-value type and it holds any values and <code>overwrite</code> is <code>true</code>
	 * all previous values are removed and the field is set to the one <code>value</code>.
	 * 
	 * @param field name of field
	 * @param value string to be set for the field.
	 * @param overwrite if false, the value are only set if the field has no existing value
	 */
	public void setFieldValue(String field, String value, boolean overwrite)
	{
        I_Field f = getFieldByName(field);
		if(f != null) {
			int card = f.cardinality();
			if(card == I_Field.CARDINALITY_NONE )
				return;
			if(overwrite || f.getValue()==null)
				f.setValue(value);
		}
	}
	
	public String getFieldValue(String field)
	{
        I_Field f = getFieldByName(field);
		if(f != null) {
			int card = f.cardinality();
			if(card == I_Field.CARDINALITY_NONE )
				return null;
			else
				return f.getValue();
		}
		return null;
	}
	
	public int validateValues() {
		int errorCnt = 0;
		Iterator it = fields.iterator();
		while(it.hasNext()){
			I_Field f = (I_Field)it.next();
			int card = f.cardinality();
			if(card == I_Field.CARDINALITY_NONE)
				continue;
			f.clearErrorMessages();
			errorCnt += f.validate();
		}
		return errorCnt;
	}
	
	public List collectErrorMessages() {
		List l = new ArrayList();
		Iterator it = fields.iterator();
		while(it.hasNext()){
			I_Field f = (I_Field)it.next();
			int card = f.cardinality();
			if(card == I_Field.CARDINALITY_NONE)
				continue;
			List e = f.getErrorMessages();
			if(e != null)
				l.addAll(e);
		}
		return l;
	}
	
	//
	// AbstractRecords
	//
	
	/**
	 * Set values in fields on form by taking the value from matching field in record. Form fields without
	 * matching record field are not set. Neither are the fields set of <code>overwrite</code> is <code>false</code>
	 * and the field value is not null.
	 * 
	 * @param record
	 * @param overwrite
	 */
	public void populateFromRecord(I_Record record, boolean overwrite) {
		Iterator it = fields.iterator();
		while(it.hasNext()){
			I_Field f = (I_Field)it.next();
			int card = f.cardinality();
			String name = null;
			if(card == I_Field.CARDINALITY_NONE || (name = f.getName()) == null)
				continue;
			RecordField rf = record.factory().getField(name);
			if(rf == null)
				continue;
			String value = record.stringValueForField(rf);
			if(overwrite || f.getValue()==null)
				f.setValue(value);
		}
	}
	
	/**
	 * Take values from field in form and store them in fields in the record. The fields
	 * must have exactly the same name on record as in form for this to work. Fields
	 * in form that have no matching field in record is ignored.
	 * 
	 * @param record
	 * @throws ParseException
	 */
	public void pushToRecord(I_Record record) throws ParseException {
		Iterator it = fields.iterator();
		while(it.hasNext()){
			I_Field f = (I_Field)it.next();
			int card = f.cardinality();
			String name = null;
			if(card == I_Field.CARDINALITY_NONE || (name = f.getName()) == null)
				continue;
			String value = f.getValue();
			RecordField rf = record.factory().getField(name);
			if(rf != null)
				record.takeStringValueForField(rf, value);
			// else silently ignore form fields not known to the record
		}
	}
 
	//
	// Stuff
	//
	
	public String toString() {
		return toXML(true);
	}
	
	public String toXML(boolean inclVal) {
		StringBuffer b = new StringBuffer();
		b.append("\n<"+FBTAG_FORM+">");
		if(getIdent()!=null)
			b.append("\n"+PP_INDENT+"<"+FBTAG_IDENT+">" + getIdent() + "</"+FBTAG_IDENT+">");
		if(getFields() != null) {
			b.append("\n"+PP_INDENT+"<"+FBTAG_FIELDS+">");
			Iterator it = getFields().iterator();
			while(it.hasNext()){
				I_Field f = (I_Field)it.next();
				b.append(f.toXML(PP_INDENT+PP_INDENT, true));
			}
			b.append("\n"+PP_INDENT+"</"+FBTAG_FIELDS+">");
		}
		if(getLayout()!=null)
			b.append(getLayout().toXML("  "));
		if(getParamEnc()!=null)
		b.append("\n"+PP_INDENT+"<"+FBTAG_PARAM_ENC+">" + getParamEnc() + "</"+FBTAG_PARAM_ENC+">");
		b.append("\n</"+FBTAG_FORM+">");
		return b.toString();
	}
	
	/**
	 * Dumps the form as HTML to a string
	 */
	public String toHTML() throws UnknownFieldRenderException {
		return getFormRender().render(this);
	}
	
    //
	// private
	//
	
	protected void parseForm(Node root)
	{
		Node child = root.getFirstChild();
		while( child != null) {
			short nt = child.getNodeType();
			switch(nt) {
				case Node.ELEMENT_NODE:
					String nn = child.getNodeName();
					if (nn.equals(FBTAG_IDENT)) {
						setIdent(collectText(child));
					}
					else if (nn.equals(FBTAG_READONLY)) {
						setReadOnly(Form.collectText(child));
					}
					else if (nn.equals(FBTAG_FIELDS)) {
						parseFields(child);
					}
					else if (nn.equals(FBTAG_LAYOUT)) {
						setLayout(new Layout(null, child));
					}
					else if (nn.equals(FBTAG_PARAM_ENC)) {
						setParamEnc(Form.collectText(child));
					}
					break;
				default:		
					break;
			}
			child = child.getNextSibling();
		}
	}
	
	protected void parseFields(Node fields)
	{
		NodeList nl = fields.getChildNodes();
		int len = nl.getLength();
		for(int i = 0; i < len; i++) {
			Node field = nl.item(i);
			short ft = field.getNodeType();
			switch(ft) {
				case Node.ELEMENT_NODE:
					String fn = field.getNodeName();
					if (fn.equals(FBTAG_RAW)) {
						addField(new RawField(this, field));
					}
					else if (fn.equals(FBTAG_MESSAGE)) {
						addField(new MessageField(this, field));
					}
					else if (fn.equals(FBTAG_TEXT)) {
						addField(new InputField(this, field, FBTAG_TEXT));
					}
					else if (fn.equals(FBTAG_HIDDEN)) {
						addField(new HiddenField(this, field));
					}
					else if (fn.equals(FBTAG_PASSWORD)) {
						addField(new InputField(this, field, FBTAG_PASSWORD));
					}
					else if (fn.equals(FBTAG_TEXTAREA)) {
						addField(new TextAreaField(this, field));
					}
					else if (fn.equals(FBTAG_CHECK)) {
						addField(new CheckBoxField(this, field));
					}
					else if (fn.equals(FBTAG_RADIO)) {
						addField(new RadioList(this, field));
					}
					else if (fn.equals(FBTAG_POPUP)) {
						addField(new PopUpList(this, field));
					}
					else if (fn.equals(FBTAG_PRIVATE)) {
						addField(new PrivateField(this, field));
					}
					else if (fn.equals(FBTAG_SELECTLIST)) {
						addField(new SelectList(this, field));
					}
					else if (fn.equals(FBTAG_CHECKLIST)) {
						addField(new CheckList(this, field));
					}
					else if (fn.equals(FBTAG_SUBMIT_LIST)) {
						addField(new SubmitList(this, field));
					}
					else if (fn.equals(FBTAG_SUBMIT)) {
						addField(new ButtonField(this, field, FBTAG_SUBMIT));
					}
					else if (fn.equals(FBTAG_RESET)) {
						addField(new ButtonField(this, field, FBTAG_RESET));
					}
					else if (fn.equals(FBTAG_BUTTON)) {
						addField(new ButtonField(this, field, FBTAG_BUTTON));
					}
					break;
				default:		
					break;
			}
		}
	}
}
