/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.forms;

import java.util.*;

/**
 * Minimal interface for accessing fields in a form defintion
 *
 */
public interface I_Field {

	/**
	 * Name of the tag
	 * @return
	 */
	public String tagName();
	
	public Form form();
	
	public static int CARDINALITY_NONE = 0;
	public static int CARDINALITY_ONE = 1;
	public static int CARDINALITY_MULTIPLE = 2;
	
	public boolean isReadOnly();
	
	/**
	 * The number of values related to field, CARDINALITY_NONE, CARDINALITY_ONE or CARDINALITY_MULTI.
	 * If NONE this it is a label or likewise. If ONE the control can handle one value.
	 * If MULTIPLE there can be multiple values selected by control. The latter requires more than one
	 * option to select from, while it may be the case for ONE (depending on the control)
	 * @return
	 */
	public int cardinality();
	
	/**
	 * While it may be null, all fields must define the method and it must be unique among all fields
	 * @return
	 */
	public String getIdent();
	
	/**
	 * May be null except for input fields, often unique but need not to be
	 * @return
	 */
	public String getName();
	
	/**
	 * Use cardinality() to determine the validity of results returned from the various value methods
	 * If cardinality is one or more getValues() must return a list even in case of one (returns null if 
	 * the value is null). In case of cardinality > 1 then it may return null or an empty list if it is unset.
	 * @return
	 */
	public String getValue();
	public void setValue(String value);
	public List getValues();
	public void setValues(List values);
	public void setValues(String[] values);
	
	public FieldFormat getFormat();
	
	/**
	 * Options are used by those controls that can select among more values
	 * @return
	 */
	public List getOptions();
	public void setOptions(List values);
	
	/**
	 * Returns 0 if the given value is valid otherwise a positive number indicating number of errors found
	 * @param value
	 * @return
	 */
	public int validate();
	
	/**
	 * Returns a list of strings indicating the errors found by validate
	 * @return
	 */
	public List getErrorMessages();
	public void clearErrorMessages();
	public void addErrorMessage(String msg);
	
	/**
	 * Generate form using the default renderer
	 * @param indent
	 * @return
	 */
	public String toHTML() throws UnknownFieldRenderException;
	
	/**
	 * Generate XML
	 * @param indent
	 * @param includeValidators
	 * @return
	 */
	public String toXML(String indent, boolean includeValidators);
}
