/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import org.w3c.dom.*;

/**
 *
 */
public class Layout {
	
	// TODO consider the place for these defaults now that render has been made a separate piece of functionality (each render could have
	// its own default layout)
	private static String _FormAction = null;
	private static String _FormMethod = "post";
	private static String _FormName = null;
	private static String _FormBegin = "\n<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">";
	private static String _FormEnd = "\n</table>";
	private static String _LabelBegin = "\n<tr><td class=\"Label\">";
	private static String _LabelEnd = "</td><td width=\"10\"></td>";
	private static String _FieldBegin = "\n<td class=\"Field\">";
	private static String _FieldEnd = "</td></tr>";
	private static String _OptionBegin = "";
	private static String _OptionEnd = "&nbsp;";
	private static String _PostscriptBegin = "";
	private static String _PostscriptEnd = "";
	private static String _StylePrefix = null;
	private static String _StyleName = null;
	private static String _Style = null;
	private static String _Rows = "4";
	private static String _Cols = null;
	
	//
	// instance
	//

	Form inheritFrom;
	
	private String styleName;
	private String style;
	private String formAction;
	private String formMethod;
	private String formName;
	private String formBegin;
	private String formEnd;
	private String labelBegin;
	private String labelEnd;
	private String fieldBegin;
	private String fieldEnd;
	private String optionBegin;
	private String optionEnd;
	private String postscriptBegin;
	private String postscriptEnd;
	private String stylePrefix;
	private String rows;
	private String cols;
    	
	public Layout(Form containingForm) {
		super();
		this.inheritFrom = containingForm;
	}
	
	public Layout(Form containingForm, Node node) {
		super();
		this.inheritFrom = containingForm;
		populateFromNode(node);
	}
	
	public void populateFromNode(Node node) 
	{
		NodeList list = node.getChildNodes();
		int len = list.getLength();
		for(int i = 0; i < len; i++) {
			Node c = list.item(i);
			if(c.getNodeType() == Node.ELEMENT_NODE){
				if(Form.FBTAG_STYLE_NAME.equals(c.getNodeName()))
					setStyleName(Form.collectText(c,""));
				else if(Form.FBTAG_STYLE.equals(c.getNodeName()))
					setStyle(Form.collectText(c,""));
				else if(Form.FBTAG_FORM_ACTION.equals(c.getNodeName()))
					setFormAction(Form.collectText(c,""));
				else if(Form.FBTAG_FORM_METHOD.equals(c.getNodeName()))
					setFormMethod(Form.collectText(c,""));
				else if(Form.FBTAG_FORM_NAME.equals(c.getNodeName()))
					setFormName(Form.collectText(c,""));
				else if(Form.FBTAG_FORM_BEGIN.equals(c.getNodeName()))
					setFormBegin(Form.collectText(c,""));
				else if(Form.FBTAG_FORM_END.equals(c.getNodeName()))
					setFormEnd(Form.collectText(c,""));
				else if(Form.FBTAG_LABEL_BEGIN.equals(c.getNodeName()))
					setLabelBegin(Form.collectText(c,""));
				else if(Form.FBTAG_LABEL_END.equals(c.getNodeName()))
					setLabelEnd(Form.collectText(c,""));
				else if(Form.FBTAG_FIELD_BEGIN.equals(c.getNodeName()))
					setFieldBegin(Form.collectText(c,""));
				else if(Form.FBTAG_FIELD_END.equals(c.getNodeName()))
					setFieldEnd(Form.collectText(c,""));
				else if(Form.FBTAG_OPTION_BEGIN.equals(c.getNodeName()))
					setOptionBegin(Form.collectText(c,""));
				else if(Form.FBTAG_OPTION_END.equals(c.getNodeName()))
					setOptionEnd(Form.collectText(c,""));
				else if(Form.FBTAG_POSTSCRIPT_BEGIN.equals(c.getNodeName()))
					setPostscriptBegin(Form.collectText(c,""));
				else if(Form.FBTAG_POSTSCRIPT_END.equals(c.getNodeName()))
					setPostscriptEnd(Form.collectText(c,""));
				else if(Form.FBTAG_STYLE_PREFIX.equals(c.getNodeName()))
					setStylePrefix(Form.collectText(c,""));
				else if(Form.FBTAG_ROWS.equals(c.getNodeName()))
					setRows(Form.collectText(c,""));
				else if(Form.FBTAG_COLS.equals(c.getNodeName()))
					setCols(Form.collectText(c,""));
			}
		}
	}	
	
	//
	// accessors
	//

	public void setStyleName(String val){
		styleName = val;
	}
	public String getStyleName() {
		if(styleName == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getStyleName()) != null)
				return tmp;
			else
				return _StyleName;
		}
		return styleName;	
	}
	public String styleName() {
		String sty = getStyleName();
		if(sty == null)
			return null;
		String pre = getStylePrefix();
		if(pre == null)
			return sty;
		else
			return pre + sty;
	}
	
	public void setStyle(String val){
		style = val;
	}
	public String getStyle() {
		if(style == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getStyle()) != null)
				return tmp;
			else
				return _Style;
		}
		return style;	
	}
	
	public void setFormAction(String val) {
		formAction = val;
	}
	public String getFormAction() {
		if(formAction == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getFormAction()) != null)
				return tmp;
			else
				return _FormAction;
		}
		return formAction;
	}
	
	public void setFormMethod(String val) {
		formMethod = val;
	}
	public String getFormMethod() {
		if(formMethod == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getFormMethod()) != null)
				return tmp;
			else
				return _FormMethod;
		}
		return formMethod;	
	}
	
	public void setFormName(String val){
		formName = val;
	}
	public String getFormName() {
		if(formName == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getFormName()) != null)
				return tmp;
			else
				return _FormName;
		}
		return formName;		
	}
	
	public void setFormBegin(String val){
		formBegin = val;
	}
	public String getFormBegin() {
		if(formBegin == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getFormBegin()) != null)
				return tmp;
			else
				return _FormBegin;
		}
		return formBegin;		
	}
	
	public void setFormEnd(String val){
		formEnd = val;
	}
	public String getFormEnd() {
		if(formEnd == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getFormEnd()) != null)
				return tmp;
			else
				return _FormEnd;
		}
		return formEnd;	
	}
	
	public void setLabelBegin(String val){
		labelBegin = val;
	}
	public String getLabelBegin() {
		if(labelBegin == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getLabelBegin()) != null)
				return tmp;
			else
				return _LabelBegin;
		}
		return labelBegin;		
	}
	
	public void setLabelEnd(String val){
		labelEnd = val;
	}
	public String getLabelEnd() {
		if(labelEnd == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getLabelEnd()) != null)
				return tmp;
			else
				return _LabelEnd;
		}
		return labelEnd;		
	}
	
	public void setFieldBegin(String val){
		fieldBegin = val;
	}
	public String getFieldBegin() {
		if(fieldBegin == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getFieldBegin()) != null)
				return tmp;
			else
				return _FieldBegin;
		}
		return fieldBegin;	
	}
	
	public void setFieldEnd(String val){
		fieldEnd = val;
	}
	public String getFieldEnd() {
		if(fieldEnd == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getFieldEnd()) != null)
				return tmp;
			else
				return _FieldEnd;
		}
		return fieldEnd;		
	}
	
	public void setOptionBegin(String val){
		optionBegin = val;
	}
	public String getOptionBegin() {
		if(optionBegin == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getOptionBegin()) != null)
				return tmp;
			else
				return _OptionBegin;
		}
		return optionBegin;		
	}
	
	public void setOptionEnd(String val){
		optionEnd = val;
	}
	public String getOptionEnd() {
		if(optionEnd == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getOptionEnd()) != null)
				return tmp;
			else
				return _OptionEnd;
		}
		return optionEnd;		
	}
	
	public void setPostscriptBegin(String val){
		postscriptBegin = val;
	}
	public String getPostscriptBegin() {
		if(postscriptBegin == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getPostscriptBegin()) != null)
				return tmp;
			else
				return _PostscriptBegin;
		}
		return postscriptBegin;		
	}
	
	public void setPostscriptEnd(String val){
		postscriptEnd = val;
	}
	public String getPostscriptEnd() {
		if(postscriptEnd == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getPostscriptEnd()) != null)
				return tmp;
			else
				return _PostscriptEnd;
		}
		return postscriptEnd;		
	}
	
	public void setRows(String val){
		rows = val;
	}
	public String getRows() {
		if(rows == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getRows()) != null)
				return tmp;
			else
				return _Rows;
		}
		return rows;		
	}

	public void setCols(String val){
		cols = val;
	}
	public String getCols() {
		if(cols == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getCols()) != null)
				return tmp;
			else
				return _Cols;
		}
		return cols;		
	}

	public void setStylePrefix(String val){
		stylePrefix = val;
	}
	public String getStylePrefix() {
		if(stylePrefix == null) {
			String tmp = null;
			if(inheritFrom != null && inheritFrom.getLayout() != null && (tmp = inheritFrom.getLayout().getStylePrefix()) != null)
				return tmp;
			else
				return _StylePrefix;
		}
		return stylePrefix;	
	}
	
	//
	//
	//
	
	public String toXML(String indent) {
		return "\n" + indent + "<" + Form.FBTAG_LAYOUT + ">" + childrenToXML(indent + Form.PP_INDENT) + "\n" + indent + "</" + Form.FBTAG_LAYOUT + ">";
	}
	
	public String childrenToXML(String indent) 
	{
		// Don't use the accessor methods since they may return stuff belonging to parent		
		StringBuffer b = new StringBuffer();
		if(styleName != null)
			b.append("\n" + indent + "<"+Form.FBTAG_STYLE_NAME+">" + styleName + "</"+Form.FBTAG_STYLE_NAME+">");
		if(style != null)
			b.append("\n" + indent + "<"+Form.FBTAG_STYLE+">" + style + "</"+Form.FBTAG_STYLE+">");
		if(formAction != null)
			b.append("\n" + indent + "<"+Form.FBTAG_FORM_ACTION+">" + formAction + "</"+Form.FBTAG_FORM_ACTION+">");
		if(formMethod != null)
			b.append("\n" + indent + "<"+Form.FBTAG_FORM_METHOD+">" + formMethod + "</"+Form.FBTAG_FORM_METHOD+">");
		if(formName != null)
			b.append("\n" + indent + "<"+Form.FBTAG_FORM_NAME+">" + formName + "</"+Form.FBTAG_FORM_NAME+">");
		if(formBegin != null)
			b.append("\n" + indent + "<"+Form.FBTAG_FORM_BEGIN+">" + Form.cdata(formBegin) + "</"+Form.FBTAG_FORM_BEGIN+">");
		if(formEnd != null)
			b.append("\n" + indent + "<"+Form.FBTAG_FORM_END+">" + Form.cdata(formEnd) + "</"+Form.FBTAG_FORM_END+">");
		if(labelBegin != null)
			b.append("\n" + indent + "<"+Form.FBTAG_LABEL_BEGIN+">" + Form.cdata(labelBegin) + "</"+Form.FBTAG_LABEL_BEGIN+">");
		if(labelEnd != null)
			b.append("\n" + indent + "<"+Form.FBTAG_LABEL_END+">" + Form.cdata(labelEnd) + "</"+Form.FBTAG_LABEL_END+">");
		if(fieldBegin != null)
			b.append("\n" + indent + "<"+Form.FBTAG_FIELD_BEGIN+">" + Form.cdata(fieldBegin) + "</"+Form.FBTAG_FIELD_BEGIN+">");
		if(fieldEnd != null)
			b.append("\n" + indent + "<"+Form.FBTAG_FIELD_END+">" + Form.cdata(fieldEnd) + "</"+Form.FBTAG_FIELD_END+">");
		if(optionBegin != null)
			b.append("\n" + indent + "<"+Form.FBTAG_OPTION_BEGIN+">" + Form.cdata(optionBegin) + "</"+Form.FBTAG_OPTION_BEGIN+">");
		if(optionEnd != null)
			b.append("\n" + indent + "<"+Form.FBTAG_OPTION_END+">" + Form.cdata(optionEnd) + "</"+Form.FBTAG_OPTION_END+">");
		if(postscriptBegin != null)
			b.append("\n" + indent + "<"+Form.FBTAG_POSTSCRIPT_BEGIN+">" + Form.cdata(postscriptBegin) + "</"+Form.FBTAG_POSTSCRIPT_BEGIN+">");
		if(postscriptEnd != null)
			b.append("\n" + indent + "<"+Form.FBTAG_POSTSCRIPT_END+">" + Form.cdata(postscriptEnd) + "</"+Form.FBTAG_POSTSCRIPT_END+">");
		if(stylePrefix != null)
			b.append("\n" + indent + "<"+Form.FBTAG_STYLE_PREFIX+">" + stylePrefix + "</"+Form.FBTAG_STYLE_PREFIX+">");
		if(rows != null)
			b.append("\n" + indent + "<"+Form.FBTAG_ROWS+">" + rows + "</"+Form.FBTAG_ROWS+">");
		if(cols != null)
			b.append("\n" + indent + "<"+Form.FBTAG_COLS+">" + style + "</"+Form.FBTAG_COLS+">");
		return b.toString();
	}
}
