/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 182 $
 * $LastChangedDate: 2007-07-05 11:13:28 +0200 (Thu, 05 Jul 2007) $
 * 
 */

package com.codedroids.j;

import java.util.List;
import java.util.ArrayList;

public class LevelNumbering {

	private List numbering = new ArrayList();
	private int currentLevel;
	private int currentNumber;
	private String separator = "_";
	
	public LevelNumbering() {
		this("_");
	}
	
	public LevelNumbering(String sep) {
		setCurrent(0, 0);
		separator = sep;
	}

	
	public void setCurrent(int level, int number) {
		// add the missing levels (setting their numbers to 0.)
		while(level >= numbering.size()) {
			numbering.add(new Integer(0));
		}
		numbering.set(level, new Integer(number));
		currentLevel = level;
		currentNumber = number;
	}
	
	public int currentLevel() {
		return currentLevel;
	}
	
	public int currentNumber() {
		return currentNumber;
	}
	
	public int incrementLevel() {
		setCurrent(++currentLevel, 0);
		return currentLevel;
	}
	
	public int decrementLevel() {
		setCurrent(--currentLevel, 0);
		return currentLevel;
	}

	public int incrementNumber() {
		setCurrent(currentLevel, ++currentNumber);
		return currentNumber;
	}
	
	public int decrementNumber() {
		setCurrent(currentLevel, --currentNumber);
		return currentNumber;
	}
	
	public void setNumber(int n) {
		setCurrent(currentLevel, n);
	}
	
	public String toString() {
		  StringBuffer id = new StringBuffer();
		  for(int i = 0; i<=currentLevel; i++) {
		    id.append(separator);
		    id.append(numbering.get(i).toString());
		  }
		  return id.toString();
		}

}
