/*
 * Copyright (c) 2006-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.oc;

import org.opencms.jsp.CmsJspBean;
import org.opencms.main.CmsException;

public interface I_OCCacheElement {

	/**
	 * <p>The OCCache calls this method after creating a new instance. The context is the one provided when the
	 * element is first cached, hence the OCCache does not make any assumptions about this value. This is left
	 * entirely to the particular element classes.</p>
	 * 
	 * @param key The key under which the element is cached. Other than it must be unique, OCCache makes no assumptions about the key.
	 * @param online Indicates whether element is cached in the online or offline part of the cache
	 * @param context Optional context, may be null. Implementations must define their own interpretation.
	 * @return If this returns false the element will not be  stored in the OCCache.
	 */
	public boolean init(CmsJspBean cms, boolean online, String key, Object context) throws CmsException;
	
	/**
	 * <p>Called by OCCache if a <code>isStale()</code> returned true.</p>
	 * @return If this returns false the element will be removed from the OCCache.
	 */
	public boolean refresh(CmsJspBean cms) throws CmsException;

	/**
	 * <p>Implementations must determine if this needs refreshing.
	 * Note that in case of the offline cache, this is only checked once the minimum offline cache period is expired for the element.
	 *  
	 * @param bestBefore
	 * @return
	 */
	public boolean isStale(CmsJspBean cms) throws CmsException;

}
