/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 173 $
 * $LastChangedDate: 2007-05-02 19:22:00 +0200 (Wed, 02 May 2007) $
 * 
 */

package com.codedroids.oc.forms;

import com.codedroids.forms.*;
import org.opencms.file.CmsObject;

/**
 * Supports a number of selectors provided by OpenCms, the selected value is then inserted into
 * the given input field. Concrete classes provide specific constructor as well as widget render.
 */
public abstract class A_OCSeletorInputFieldRender implements I_FieldRender {

	CmsObject cms;
	
	public A_OCSeletorInputFieldRender(CmsObject cms) {
		super();
		this.cms = cms;
	}

	public String render(I_Field field) throws UnknownFieldRenderException {
		if(field instanceof InputField) {
			return renderInputField((InputField)field);
		}
		else {
			throw new UnknownFieldRenderException(this.getClass().getName() +  " cannot render field: " + field.tagName());
		}
	}

	public String renderInputField(InputField f) {
		StringBuffer b = new StringBuffer(f.getLayout().getLabelBegin());
		if(f.getLabel() != null) {
			b.append("<label" + (f.getIdent() != null ? " for=\"" + f.getIdent() + "\"" : "") + ">");
			b.append(f.getLabel());
			b.append("</label>");
		}
		b.append(f.getLayout().getLabelEnd());
		b.append(f.getLayout().getFieldBegin());
		b.append("<input");
		b.append(" type=\"" + f.type() + "\"");
		b.append(" name=\"" + f.getName() + "\"");
		b.append(" id=\"" + f.getIdent() + "\"");
		if(f.getValue() != null) {
			b.append(" value=\"" + Form.escapeAttribute(f.getValue()) + "\"");
		}
		if(f.isReadOnly()) {
			if(f.type().equals("text")) {
				b.append(" readonly");
			} else {
				b.append(" disabled");
			}
		}
		
		if(f.getLayout().getCols() != null)
			b.append(" size=\"" + f.getLayout().getCols() + "\"");
		b.append(" class=\"" + f.tagName());
		if(f.isReadOnly())
			b.append(" readonly");
		if(f.getLayout().styleName() != null)
			b.append(" " + f.getLayout().styleName());
		b.append("\"");
		if(f.getLayout().getStyle() != null)
			b.append(" style=\"" + f.getLayout().getStyle() + "\"");
		b.append(">");
	    b.append(f.getLayout().getPostscriptBegin());
		b.append(renderWidget(f));
		if(f.getPostscript()!=null)
			b.append(f.getPostscript());
		b.append(f.getLayout().getPostscriptEnd());
		b.append(f.getLayout().getFieldEnd());
		return b.toString();
	}
	
	abstract public String renderWidget(InputField f); 
}
