/*
 * Copyright (c) 2006-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 120 $
 * $LastChangedDate: 2007-02-23 15:39:39 +0100 (Fri, 23 Feb 2007) $
 * 
 */

package com.codedroids.oc.templating;

import com.codedroids.templating.A_TemplateEngine;
import com.codedroids.templating.TemplateEngineFactory;
import org.opencms.jsp.CmsJspActionElement;
import java.util.*;
import com.codedroids.j.*;
import net.sf.json.JSONObject;

public class OCTemplateEngineFactory extends TemplateEngineFactory {

	/**
	 * Reserved key stating the (class) name of the engine to be instantiated by the template engine factory
	 */
	public static final String TEF_CONFIG_ENGINE = "engine";
	
	/**
	 * Reserved key that holds the name of the template which the template engines probably needs 
	 */
	public static final String TEF_CONFIG_TEMPLATE = "template";


	// Eventually we should get these from module config or the like...
	static {
		// Note that this also register the engine under its class name, so no need to explicitly register the class names as well 
		TemplateEngineFactory.registerTemplateEngineClassName("com.codedroids.fm.templating.FMTemplateEngine", "ftl");
		TemplateEngineFactory.registerTemplateEngineClassName("com.codedroids.fm.templating.FMTemplateEngine", "freemarker");
		TemplateEngineFactory.registerTemplateEngineClassName("com.codedroids.oc.templating.XSLTTemplateEngine", "xslt");
		TemplateEngineFactory.registerTemplateEngineClassName("com.codedroids.oc.templating.OCTemplateEngine", "macro");
	}

	public static A_TemplateEngine templateEngine(CmsJspActionElement cms, String templateName, String configStr) 
	throws InstantiationException, IllegalAccessException, ClassNotFoundException
	{
		String className = null;
		JSONObject config = null;
		
		if(!configStr.trim().startsWith("{")) {
			// 'Old-style' config string, this means there must be one of the following substrings:
			// type:ftl or type:xslt - map them to their respective classes.
			// Format is CSS like: "type:ftl;some:value..."
			Map settings = MapUtils.populateMapFromString(new HashMap(), configStr);
			String type = (String)settings.get("type");
			if(type == null)
				type = typeFromTemplateName(templateName);
			if(type != null)
				className = TemplateEngineFactory.templateEngineClassName(type);
			if(className != null) {
				config = JSONObject.fromMap(settings);
			}
		} else {
			// JSON format
			config = JSONObject.fromString(configStr);
			String eng = config.getString(TEF_CONFIG_ENGINE);
			if(eng == null)
				eng = typeFromTemplateName(templateName);
			if(eng != null)
				className = TemplateEngineFactory.templateEngineClassName(eng);
		}
		
		return templateEngine(cms, className, templateName, config);
	}
	
	/**
	 * Returns a configured (init'ed) template engine given by the className
	 * 
	 * @param cms
	 * @param className
	 * @param templateName
	 * @param config
	 * @return
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 */
	public static A_TemplateEngine templateEngine(CmsJspActionElement cms, String className, String templateName, JSONObject config) 
	throws InstantiationException, IllegalAccessException, ClassNotFoundException
	{
		// TODO could cache on the templateName (plus path) and config.toString()
		if(className == null)
			throw new ClassNotFoundException("Invalid parameter - className was null");
		A_OCTemplateEngine instance = (A_OCTemplateEngine)Class.forName(className).newInstance();
		// Set up the configuration for the templat engine instance
		config.put(TEF_CONFIG_ENGINE, className); // set the actual class name
		config.put(TEF_CONFIG_TEMPLATE, templateName); // template engine knows what to do with it
		//config.put(TEF_CONFIG_CMS, cms); // JSOn chokes on this - anyway, it is an explicit parameter now
		instance.setConfiguration(config);
		instance.setCms(cms);
		instance.init();
		return instance;
	}
	
	
	/**
	 * Convenience in case no engine is given by the configuration
	 * 
	 * @param templateName
	 * @return
	 */
	private static String typeFromTemplateName(String templateName) {
		if(templateName.endsWith(".ftl"))
			return "ftl";
		else if(templateName.endsWith(".xslt") || templateName.endsWith(".xsl"))
			return "xslt";
		else
			return null;
	}
	
}
