/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.records;

import java.util.*;

public class GenericClause extends A_Clause {

	RecordField field;
	String operator;
	Object value;
	
	public GenericClause() {
		super();
	}
	
	public GenericClause(RecordFactory factory, String field, String operator, Object value) {
		this(factory.getField(field), operator, value);
	}
	
	public GenericClause(RecordField field, String operator, Object value) {
		this();
		init(field, operator, value);
	}

	public void init(RecordField field, String operator, Object value) {
		this.field = field;
		this.operator = operator;
		this.value = value;
	}
	
	// returns the number of parameters this query operand takes;
	public int cardinality() {
		return 1;
	}
	
	public Object getValue() {
		return value;
	}
	
	public List getValues() {
		return Collections.singletonList(value);
	}
	
	public RecordField getField() {
		return field;
	}
	
	public List getFields() {
		return Collections.singletonList(field);
	}
	
	public String toSQL() {
		return field.getColumn() + " " + operator + " ?";
	}
}
